Package: ranger
Type: Package
Title: A Fast Implementation of Random Forests
Version: 0.7.0
Date: 2017-03-23
Author: Marvin N. Wright
Maintainer: Marvin N. Wright <wright@imbs.uni-luebeck.de>
Description: A fast implementation of Random Forests, particularly suited for high
          dimensional data. Ensembles of classification, regression, survival and
          probability prediction trees are supported. Data from genome-wide association
          studies can be analyzed efficiently. In addition to data frames, datasets of
          class 'gwaa.data' (R package 'GenABEL') can be directly analyzed.
License: GPL-3
Imports: Rcpp (>= 0.11.2)
LinkingTo: Rcpp
Depends: R (>= 3.1)
Suggests: survival, testthat, GenABEL
RoxygenNote: 5.0.1
URL: https://github.com/imbs-hl/ranger
BugReports: https://github.com/imbs-hl/ranger/issues
NeedsCompilation: yes
Packaged: 2017-03-23 13:33:07 UTC; wright
Repository: CRAN
Date/Publication: 2017-03-31 16:19:06 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-01-23 13:43:42 UTC; windows
Archs: x64
