# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,ranger.prediction)
S3method(deforest,ranger)
S3method(importance,ranger)
S3method(predict,ranger)
S3method(predict,ranger.forest)
S3method(predictions,ranger)
S3method(predictions,ranger.prediction)
S3method(print,deforest.ranger)
S3method(print,ranger)
S3method(print,ranger.forest)
S3method(print,ranger.prediction)
S3method(str,ranger)
S3method(str,ranger.forest)
S3method(timepoints,ranger)
S3method(timepoints,ranger.prediction)
export(csrf)
export(deforest)
export(getTerminalNodeIDs)
export(holdoutRF)
export(hshrink)
export(importance)
export(importance_pvalues)
export(predictions)
export(ranger)
export(timepoints)
export(treeInfo)
import(stats)
import(utils)
importFrom(Matrix,Matrix)
importFrom(Rcpp,evalCpp)
useDynLib(ranger, .registration = TRUE)
